import * as ol from 'ol'
import * as style from 'ol/style'
import * as layer from 'ol/layer'
import * as source from 'ol/source'
import * as geom from 'ol/geom'
import * as proj from 'ol/proj'
import * as interaction from 'ol/interaction'
import * as coordinate from 'ol/coordinate'
import * as format from "ol/format";
import {fromExtent} from 'ol/geom/Polygon';
// @ts-ignore
import LocationImg from '../images/location.png'
import {ElMessage} from "element-plus";

const layerFlag = ['layerName', 'positionLayer']
export let locationTooltipElement
export default function Location ({map, position = null, wkt = null, zoom = null, color = '#039ff3'}) {
    try {
        const feat: any = new format.WKT().readFeature(position ? `POINT(${position[0]} ${position[1]})` : wkt)
        const mapViewExtent = map.getView().get('extent')
        if (mapViewExtent) {
            const poly = fromExtent(mapViewExtent)
            if (!poly.intersectsCoordinate(feat.getGeometry().getCoordinates())) {
                ElMessage.warning('超出范围，无法定位该位置!')
                return
            }
        }
        let _source
        const realLayer = map.getLayers().getArray().filter(v => v.get(layerFlag[0]) === layerFlag[1])
        if (realLayer[0]) {
            _source = realLayer[0].getSource()
            _source.clear()
        } else {
            _source = new source.Vector(); //图层数据源
            const _vector = new layer.Vector({
                zIndex: 9999,
                source: _source,
            });
            _vector.set(layerFlag[0], layerFlag[1])
            map.addLayer(_vector);
        }
        let locationTooltip
        const id = 'locationTooltipElementId'
        if (locationTooltipElement) {
            map.removeOverlay(map.getOverlayById(id))
            locationTooltipElement.parentNode.removeChild(locationTooltipElement);
        }
        locationTooltipElement = document.createElement('div');
        locationTooltipElement.className = 'tooltip-location';
        locationTooltipElement.onclick = () => {
            _source.removeFeature(feat)
            map.removeOverlay(map.getOverlayById(id))
            locationTooltipElement.parentNode.removeChild(locationTooltipElement);
            locationTooltipElement = null
        };
        locationTooltip = new ol.Overlay({
            id,
            element: locationTooltipElement,
            offset: [0, -15],
            positioning: 'bottom-center'
        });
        map.addOverlay(locationTooltip);
        const radius = 25
        const longRadius = radius * Math.SQRT2
        feat.setStyle([new style.Style({ //图层样式
            image: new style.Icon({
                src: LocationImg,
                displacement:[0, 15]
            })
        })])
        _source.addFeature(feat)
        locationTooltip.setPosition(feat.getGeometry().getCoordinates())
        // setTimeout(() => {
        //     // @ts-ignore
        //     feat?.getGeometry().setCoordinates([0, 0])
        // }, 3000)
        // @ts-ignore
        map.getView().setCenter(feat?.getGeometry().getCoordinates())
        if (zoom) {
            map.getView().setZoom(zoom)
        }
    }catch (e) {
        console.log(wkt ?? position)
        ElMessage.warning('坐标异常！')
    }
}

export const clearLocationDom = (map) => {
    if (locationTooltipElement) {
        locationTooltipElement.parentNode.removeChild(locationTooltipElement);
        locationTooltipElement = null
    }
    map.getLayers().getArray().filter(v => v.get(layerFlag[0]) === layerFlag[1])?.[0]?.getSource()?.clear()
}